/*******************************************************************************
 * Copyright (c) 2000, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     yidabu at gmail dot com  ( D China http://www.d-programming-language-china.org/ )
 *******************************************************************************/

module styledtext.Snippet217;
/**
 * StyledText snippet: embed controls
 *
 * For a list of all SWT example snippets see
 * http://www.eclipse.org/swt/snippets/
 *
 * @since 3.2
 */

import dwt.DWT;
import dwt.custom.StyledText;
import dwt.custom.StyleRange;
import dwt.custom.PaintObjectEvent;
import dwt.custom.PaintObjectListener;
import dwt.layout.GridData;
import dwt.layout.GridLayout;
import dwt.widgets.Display;
import dwt.widgets.Shell;
import dwt.widgets.Control;
import dwt.widgets.Button;
import dwt.widgets.Combo;
import dwt.widgets.Event;
import dwt.widgets.Listener;
import dwt.graphics.FontData;
import dwt.graphics.Font;
import dwt.graphics.Rectangle;
import dwt.graphics.Point;
import dwt.graphics.GC;
import dwt.graphics.Image;
import dwt.graphics.GlyphMetrics;

import dwt.dwthelper.utils;
version(JIVE){
    import jive.stacktrace;
}

void main() {
    static StyledText styledText;
    static String text =
        "This snippet shows how to embed widgets in a StyledText.\n"
        "Here is one: \uFFFC, and here is another: \uFFFC.";
    static int[] offsets;
    static Control[] controls;
    static int MARGIN = 5;

    void addControl(Control control, int offset) {
        StyleRange style = new StyleRange ();
        style.start = offset;
        style.length = 1;
        control.pack();
        Rectangle rect = control.getBounds();
        int ascent = 2*rect.height/3;
        int descent = rect.height - ascent;
        style.metrics = new GlyphMetrics(ascent + MARGIN, descent + MARGIN, rect.width + 2*MARGIN);
        styledText.setStyleRange(style);
    }


    Display display = new Display();
    Font font = new Font(display, "Tahoma", 32f, DWT.NORMAL);
    Shell shell = new Shell(display);
    shell.setLayout(new GridLayout());
    styledText = new StyledText(shell, DWT.WRAP | DWT.BORDER);
    styledText.setFont(font);
    styledText.setLayoutData(new GridData(DWT.FILL, DWT.FILL, true, true));
    styledText.setText(text);
    controls = new Control[2];
    Button button = new Button(styledText, DWT.PUSH);
    button.setText("Button 1");
    controls[0] = button;
    Combo combo = new Combo(styledText, DWT.NONE);
    combo.add("item 1");
    combo.add("another item");
    controls[1] = combo;
    offsets = new int[controls.length];
    int lastOffset = 0;
    for (int i = 0; i < controls.length; i++) {
        int offset = text.indexOf("\uFFFC", lastOffset);
        offsets[i] = offset;
        addControl(controls[i], offsets[i]);
        lastOffset = offset + 1;
    }

    void onVerify(Event e) {
        int start = e.start;
        int replaceCharCount = e.end - e.start;
        int newCharCount = e.text.length;
        for (int i = 0; i < offsets.length; i++) {
            int offset = offsets[i];
            if (start <= offset && offset < start + replaceCharCount) {
                // this widget is being deleted from the text
                if (controls[i] !is null && !controls[i].isDisposed()) {
                    controls[i].dispose();
                    controls[i] = null;
                }
                offset = -1;
            }
            if (offset != -1 && offset >= start) offset += newCharCount - replaceCharCount;
            offsets[i] = offset;
        }
    }
    // use a verify listener to keep the offsets up to date
    styledText.addListener(DWT.Verify, dgListener(&onVerify));

    // reposition widgets on paint event
    styledText.addPaintObjectListener(new class(offsets) PaintObjectListener {
        int[] offsets;
        this(int[] offsets_) {
            this.offsets = offsets_;
        }
        public void paintObject(PaintObjectEvent event) {
            StyleRange style = event.style;
            int start = style.start;
            for (int i = 0; i < offsets.length; i++) {
                int offset = offsets[i];
                if (start == offset) {
                    Point pt = controls[i].getSize();
                    int x = event.x + MARGIN;
                    int y = event.y + event.ascent - 2*pt.y/3;
                    controls[i].setLocation(x, y);
                    break;
                }
            }
        }
    });

    shell.setSize(400, 400);
    shell.open();
    while (!shell.isDisposed()) {
        if (!display.readAndDispatch())
            display.sleep();
    }
    font.dispose();
    display.dispose();
}